int pg_db_lo_open	();
int pg_db_lo_close	();
int pg_db_lo_read	();
int pg_db_lo_write	();
int pg_db_lo_lseek	();
unsigned int pg_db_lo_creat	();
int pg_db_lo_tell	();
int pg_db_lo_unlink	();
unsigned int pg_db_lo_import	();
int pg_db_lo_export	();
int pg_db_putline	();
int pg_db_getline	();
int pg_db_endcopy	();
int dbd_db_ping		();
